<?php
# This file is distributed under GNU GPLv1 and further versions.
# This file was written by Zdenek Styblik
# This file is part of weathermap4rrd-ng
#
# Zdenek Styblik
# e-mail: zdenek <dot> styblik <at> gmail <dot> com
#
# Last update: 2008-07-23
#
class config_file {
	# default variables
	const ARROW_WIDTH = 4; # arrow width
	const DRAWGRID = 1; # draw grid(yes)
	const FONT = 3; # font size
	const HEIGHT = 480; # default image height
	const INODES = 5; # inodes to edit
	const REFRESH = 60; # refresh map every 60s
	const ROLLOVERLIB = "overlib"; # type of javascript library used of rollover above map elements
	const RRDSTEP = 300; # == 5 minutes
	const RRDTOOL = '/usr/sbin/rrdtool'; # path to rrdtool; probably not used;
	const WIDTH = 640; # default image width
	// xml-2-obj
	public $config;
	// default config file
	public $configfile = 'weathermap.xml';
	// xpath thru xml version of config file
	private $xpath;
	// init
	function __construct($file = null) {
		if (!is_null($file) && !empty($file)) {
			$this->configfile = $file;
		}
		$this->auto_index = -1;
		$this->config = new StdClass;
		$this->config->debug = new StdClass;
	}
	// clean-up
	function __destruct() {
		unset($this->xpath);
		unset($this->config);
	}
	# desc: reads xml file defined by $this->configfile;
	# @return: bit;
	public function readconfig() {
		if (file_exists("{$this->configfile}") && is_readable("{$this->configfile}")) {
			$doc = new DOMDocument('1.0', 'utf-8');
			$doc->formatOutput = true;
			$doc->load($this->configfile);
			$this->xpath = new DOMXPath($doc);
			return true;
		} else {
			return false;
		}
	}
	# desc: save xml config
	# @return: void;
	public function writeconfig($xml) {
		$doc = new DOMDocument('1.0', 'utf-8');
		$doc->formatOutput = true;
		$doc->loadXML($xml);
		$doc->save($this->configfile);
	}
	# @return: const(RRDSTEP);
	public function getRRDStep() {
		return self::RRDSTEP;
	}
	# @return: const(ARROW_WIDTH);
	public function getArrowWidth() {
		return self::ARROW_WIDTH;
	}
	# @return: const(INODES);
	public function getINODESCount() {
		return self::INODES;
	}
	# @return: const(FONT);
	public function getFontSize() {
		return self::FONT;
	}
	# @return: const(ROLLOVERLIB)
	public function getRollOverLib() {
		return self::ROLLOVERLIB;
	}
	# @return: obj;
	public function getDefaultConfig() {
		$config = new StdClass;
		$config->arrowwidth = self::ARROW_WIDTH;
		# image size
		$config->width = self::WIDTH;
		$config->height = self::HEIGHT;
		# font
		$config->font = self::FONT;
		# legend
		$config->legend = new StdClass;
		$config->legend->x = 50;
		$config->legend->y = 80;
		$config->legend->legendstyle = "original";
		# title
		$config->title = new StdClass;
		$config->title->x = 50;
		$config->title->y = 40;
		$config->title->text = "WeatherMap4RRD-default";
		## background
		$config->title->bg = new StdClass;
		$config->title->bg->r = 255;
		$config->title->bg->g = 255;
		$config->title->bg->b = 255;
		## foreground
		$config->title->fg = new StdClass;
		$config->title->fg->r = 0;
		$config->title->fg->g = 0;
		$config->title->fg->b = 0;
		# scale
		$config->autoscale = new StdClass;
		$config->autoscale->nodivisions = 10;
		$config->autoscale->rstart = 24;
		$config->autoscale->rend = 0;
		$config->autoscale->gstart = 232;
		$config->autoscale->gend = 0;
		$config->autoscale->bstart = 2;
		$config->autoscale->bend = 0;
		$retval = & $config;
		return $retval;
	}
	# desc: fetch doggie; returns xml-in->obj;
	#	@return: $obj;
	public function getConfig() {
		$config = $this->config;
		$this->getGlobal('arrowwidth', $config);
		$this->getGlobal('background', $config);
		$this->getGlobal('drawgrid', $config);
		$this->getGlobal('font', $config);
		$this->getGlobal('height', $config);
		$this->getGlobal('htmlfile', $config);
		$this->getGlobal('ipcheck', $config);
		$this->getGlobal('outputfile', $config);
		$this->getGlobal('refresh', $config);
		$this->getGlobal('width', $config);
		$config->legend = $this->getLegend();
		$config->title = $this->getTitle();
		$config->autoscale = $this->getAutoScale();
		$config->scale = $this->getScale();
		$config->nodes = $this->getNodes();
		$config->links = $this->getLinks();
		// final check
		if ($config->scale == NULL && $config->autoscale == NULL) {
			$config->autoscale = $this->defaultAutoscale();
			unset($config->scale);
		}
		if (!isset($config->arrowwidth) || !is_numeric($config->arrowwidth))
			$config->arrowwidth = self::ARROW_WIDTH;
		if (!isset($config->drawgrid) || !is_numeric($config->drawgrid))
			$config->drawgrid = self::DRAWGRID;
		if (!isset($config->font) || !is_numeric($config->font))
			$config->font = self::FONT;
		if (!isset($config->height) || !is_numeric($config->height))
			$config->height = self::HEIGHT;
		if (!isset($config->refresh) || !is_numeric($config->refresh))
			$config->refresh = self::REFRESH;
		if (!isset($config->width) || !is_numeric($config->width))
			$config->width = self::WIDTH;
		return $config;
	}
	# desc: returns node defined right in xml tree;
	# @return: $obj;
	public function getGlobal($node = null, $obj = null) {
		$node = preg_replace('/^\s+/', '', $node);
		$node = preg_replace('/\s+$/', '', $node);
		$node = preg_replace('/\s+/', '', $node);
		if (!is_null($node) && !empty($node) && is_object($obj)) {
			$result1 = $this->xpath->query("/root/{$node}");
			foreach ($result1 as $element) {
				$obj->{$node} = $element->nodeValue;
			}
		}
		$retval = & $obj;
		return $retval;
	}
	# desc: returns links with its properties;
	# @return: $obj;
	public function getLinks() {
		// '/root/links/child::*[group="foo"]' - for the future
		$result1 = $this->xpath->query('/root/links/*');
		if ($result1->length > 0) {
			$links = new StdClass;
			foreach($result1 as $xmlnode) {
				$linkname = preg_replace('/^\s+/', '', $xmlnode->tagName);
				$linkname = preg_replace('/\s+$/', '', $linkname);
				$linkname = preg_replace('/\s+/', '', $linkname);
				if (isset($links->{$linkname})) {
					# node is duplicate - and that's not allowed
				} else {
					$links->{$linkname} = new StdClass;
					$link = $links->{$linkname};
					$link->name = $linkname;
					foreach ($this->xpath->query('*', $xmlnode) as $element) {
						switch ($element->nodeName) {
							case 'arrow':
								$link->arrow = $element->nodeValue;
								break;
							case 'bandwidth':
								if (is_numeric($link->bandwidth)) {
									$link->bandwidth = $element->nodeValue;
								} else {
									$link->bandwidth = 0;
								}
								break;
							case 'displayvalue':
								if ($element->nodeValue == 1)
									$link->displayvalue = 1;
								break;
							case 'group':
								$link->group = $element->nodeValue;
								break;
							case 'from':
								$link->from = $element->nodeValue;
								break;
							case 'inodegroup':
								$link->inodegroup = $element->nodeValue;
								break;
							case 'inpos':
								if (is_numeric($element->nodeValue))
									$link->inpos = $element->nodeValue;
								break;
							case 'internodedisplay':
								if ($element->nodeValue == 1)
									$link->internodedisplay = 1;
								break;
							case 'internodedisplaymid':
								if ($element->nodeValue == 1)
									$link->internodedisplaymid = 1;
								break;
							case 'internodedisplaynum':
								if ($element->nodeValue == 1)
									$link->internodedisplaynum = 1;
								break;
							case 'linkoverlibgraph':
								$link->linkoverlibgraph = $element->nodeValue;
								break;
							case 'linkinfourl':
								$link->linkinfourl = $element->nodeValue;
								break;
							case 'nodes':
								$link->nodes = $element->nodeValue;
							 	break;
							case 'outpos':
								if (is_numeric($element->nodeValue))
									$link->outpos = $element->nodeValue;
							 	break;
							case 'ratein':
								if (is_numeric($element->nodeValue))
									$link->ratein = $element->nodeValue;
								break;
							case 'rateout':
								if (is_numeric($element->nodeValue))
									$link->rateout = $element->nodeValue;
								break;
							case 'target':
								$link->target = $element->nodeValue;
								break;
							case 'targetin':
								$link->targetin = $element->nodeValue;
								break;
							case 'targetout':
								$link->targetout = $element->nodeValue;
								break;
							case 'to':
								$link->to = $element->nodeValue;
								break;
							case 'unit':
								$link->unit = $element->nodeValue;
								break;
						}
					}
					// now query for internodes, if any
					$result2 = $this->xpath->query('internode', $xmlnode);
					$j = $result2->length;
					for ($i = 1; $i <= $j; $i++) {
						$x = null;
						$y = null;
						foreach ($this->xpath->query("internode[{$i}]/*", $xmlnode) as $inode) {
							switch ($inode->nodeName) {
								case 'x':
									if (is_numeric($inode->nodeValue))
										$x = $inode->nodeValue;
									break;
								case 'y':
									if (is_numeric($inode->nodeValue))
										$y = $inode->nodeValue;
									break;
							} // switch
						} // foreach
						if (is_numeric($x) && is_numeric($y)) {
							$link->internode[$i] = new StdClass;
							$link->internode[$i]->x = $x;
							$link->internode[$i]->y = $y;
						}
					} // for i = 1;
					// unset
					unset($result2);
				}
			}
			unset($result1);
			return $links;
		} else {
			unset($result1);
			return NULL;
		}
	}
	# desc: returns nodes with its properties;
	# @return: $obj;
	public function getNodes() {
		$result1 = $this->xpath->query('/root/nodes/*');
		if ($result1->length > 0) {
			$nodes = new StdClass;
			foreach($result1 as $xmlnode) {
				$nodename = preg_replace('/^\s+/', '', $xmlnode->tagName);
				$nodename = preg_replace('/\s+$/', '', $nodename);
				$nodename = preg_replace('/\s+/', '_', $nodename);
				if (isset($nodes->{$nodename})) {
					# node is duplicate - and that's not allowed
				} else {
					$nodes->{$nodename} = new StdClass;
					$node = $nodes->{$nodename};
					$node->name = $nodename;
					foreach($this->xpath->query('*', $xmlnode) as $element) {
						switch ($element->nodeName) {
							case 'label':
								$node->label = $element->nodeValue;
								break;
							case 'labeltype':
								$node->labeltype = $element->nodeValue;
								break;
							case 'x':
								if (is_numeric($element->nodeValue)) {
									$node->x = $element->nodeValue;
								} else {
									$node->x = 1;
								}
								break;
							case 'y':
								if (is_numeric($element->nodeValue)) {
									$node->y = $element->nodeValue;
								} else {
									$node->y = 1;
								}
								break;
							case 'icon':
								$node->icon = $element->nodeValue;
								break;
							case 'iconx':
								if (is_numeric($element->nodeValue)) {
									$node->iconx = $element->nodeValue;
								} else {
									$node->iconx = 1;
								}
								break;
							case 'icony':
								if (is_numeric($element->nodeValue)) {
									$node->icony = $element->nodeValue;
								} else {
									$node->icony = 1;
								}
								break;
							case 'icontpt':
								$var = $element->nodeValue;
								if (is_numeric($var) && $var >= 0 && $var <= 100)
									$node->icontpt = $var;	
								break;
							case 'iconresize':
								if (is_numeric($element->nodeValue))
									$node->iconresize = $element->nodeValue;
								break;
							case 'ip':
								$node->ip = $element->nodeValue;
								break;
							case 'check':
								$node->check = $element->nodeValue;
								break;
							case 'nodeurl':
								$node->nodeurl = $element->nodeValue;
								break;
						}
					}
				}
			}
			unset($result1);
			return $nodes;
		} else {
			unset($result1);
			return NULL;
		}
	}
	# desc: returns autoscale settings;
	# @return: $obj;
	public function getAutoScale() {
		$result0 = $this->xpath->query('/root/autoscale');
		if ($result0->length > 0) {
			// set up default structure
			$autoscale = $this->defaultAutoscale();
			foreach ($result0 as $e) {
				$result1 = $this->xpath->query('*', $e);
				foreach ($result1 as $element) {
					switch ($element->tagName) {
						case 'nodivisions':
							if (is_numeric($element->nodeValue))
								$autoscale->nodivisions = $element->nodeValue;
							break;
						case 'rstart':
							if (is_numeric($element->nodeValue))
								$autoscale->rstart = $element->nodeValue;
							break;
						case 'rend':
							if (is_numeric($element->nodeValue))
								$autoscale->rend = $element->nodeValue;
							break;
						case 'gstart':
							if (is_numeric($element->nodeValue))
								$autoscale->gstart = $element->nodeValue;
							break;
						case 'gend':
							if (is_numeric($element->nodeValue))
								$autoscale->gend = $element->nodeValue;
							break;
						case 'bstart':
							if (is_numeric($element->nodeValue))
								$autoscale->bstart = $element->nodeValue;
							break;
						case 'bend':
							if (is_numeric($element->nodeValue))
								$autoscale->bend = $element->nodeValue;
							break;
					}
				}
				unset($result1);
			}
			return $autoscale;
		} else {
			return NULL;
		}
	}
	# desc: default autoscale values
	# @return: $obj;
	public function defaultAutoscale() {
		$autoscale = new StdClass;
		$autoscale->nodivisions = 10;
		$autoscale->rstart = 24;
		$autoscale->rend = 0;
		$autoscale->gstart = 232;
		$autoscale->gend = 0;
		$autoscale->bstart = 2;
		$autoscale->bend = 0;
		return $autoscale;
	}
	# desc: returns scale object;
	# @return: $obj;
	public function getScale() {
		$result1 = $this->xpath->query('/root/scale/*');
		if ($result1->length > 0) {
			$scale = new StdClass;
			for ($i = 1; $i <= $result1->length; $i++) {
				$result2 = $this->xpath->query("/root/scale/entry[{$i}]/*");
				$low = null;
				$high = null;
				$r = null;
				$g = null;
				$b = null;
				foreach ($result2 as $element) {
					switch ($element->tagName) {
						case 'low':
							if (is_numeric($element->nodeValue))
								$low = $element->nodeValue;
							break;
						case 'high':
							if (is_numeric($element->nodeValue))
								$high = $element->nodeValue;
							break;
						case 'r':
							if (is_numeric($element->nodeValue))
								$r = $element->nodeValue;
							break;
						case 'g':
							if (is_numeric($element->nodeValue))
								$g = $element->nodeValue;
							break;
						case 'b':
							if (is_numeric($element->nodeValue))
								$b = $element->nodeValue;
							break;
					}
				}
				if (!is_null($low) & !is_null($high) & !is_null($r) & !is_null($g) & !is_null($b)) {
					$scale->entry[$i] = new StdClass;
					$scale->entry[$i]->low = $low;
					$scale->entry[$i]->high = $high;
					$scale->entry[$i]->r = $r;
					$scale->entry[$i]->g = $g;
					$scale->entry[$i]->b = $b;
				}
			}
			unset($result2);
		} else {
			/* however we won't return any default static scale,
			 * because there is no reason to do so. 
			 */
			// set up defaults
			/*
				$scale->entry[0] = new StdClass;
				$scale->entry[0]->low = 0;
				$scale->entry[0]->high = 100;
				$scale->entry[0]->r = 0;
				$scale->entry[0]->g = 128;
				$scale->entry[0]->b = 255; 
		 	 */
			$scale = NULL;
		}
		unset($result1);
		return $scale;
	}
	# desc: returns title object;
	# @return: $obj;
	public function getTitle() {
		// set up default structure
		$title = new StdClass;
		$title->x = 100;
		$title->y = 200;
		$title->text = "WeatherMap-NG";
		// title background
		$title->bg = new StdClass;
		$title->bg->r = 255;
		$title->bg->g = 255;
		$title->bg->b = 255;
		// title foreground
		$title->fg = new StdClass;
		$title->fg->r = 0;
		$title->fg->g = 0;
		$title->fg->b = 0;
		$result1 = $this->xpath->query('/root/title/*');
		foreach ($result1 as $element) {
			switch ($element->tagName) {
				case 'x':
					if (is_numeric($element->nodeValue))
						$title->x = $element->nodeValue;
					break;
				case 'y':
					if (is_numeric($element->nodeValue))
						$title->y = $element->nodeValue;
					break;
				case 'text':
					$title->text = $element->nodeValue;
					break;
				case 'bg':
					$result2 = $this->xpath->query('*', $element);
					$this->colorParse($result2, $title->bg);
					unset($result2);
					break;
				case 'fg':
					$result2 = $this->xpath->query('*', $element);
					$this->colorParse($result2, $title->fg);
					unset($result2);
					break;
				default:
					break;
			}
		}
		unset($result1);
		return $title;
	}
	# desc: tied with getTitle; support function;
	# $result: DOMNodeList;
	# $obj: stdClass;
	# @return: $obj;
	private function colorParse($result, $obj) {
		foreach ($result as $element) {
			switch ($element->tagName) {
				case 'r':
					if (is_numeric($element->nodeValue))
						$obj->r = $element->nodeValue;
					break;
				case 'g':
					if (is_numeric($element->nodeValue))
						$obj->g = $element->nodeValue;
					break;
				case 'b':
					if (is_numeric($element->nodeValue))
						$obj->b = $element->nodeValue;
					break;
			}
		}
		$retval = & $obj;
		return $retval;
	}
	# desc: returns legend object;
	# @return: $obj;
	public function getLegend() {
		$legend = new StdClass;
		// set up defaults
		$legend->x = 100;
		$legend->y = 100;
		$legend->legendstyle = "original";
		$result = $this->xpath->query('/root/legend/*');
		foreach ($result as $element) {
			switch ($element->tagName) {
				case 'x':
					if (is_numeric($element->nodeValue))
						$legend->x = $element->nodeValue;
					break;
				case 'y':
					if (is_numeric($element->nodeValue))
						$legend->y = $element->nodeValue;
					break;
				case 'legendstyle':
					if ($element->nodeValue == "line")
						$legend->legendstyle = $element->nodeValue;
					break;
				default:
					break;
			}
		}
		// unset things
		unset($result);
		return $legend;
	}
	# desc: returns autoscale object;
	# @return: $obj;
	public function autoscale($autoscale = null) {
		if (is_object($autoscale)) {
			if (!isset($autoscale->nodivisions) && !is_numeric($autoscale->nodivisions)) {
				$autoscale->nodivisions = 7;
			}
			
			$dif_red=-($autoscale->rstart - $autoscale->rend);
			$dif_green=-($autoscale->gstart - $autoscale->gend);
			$dif_blue=-($autoscale->bstart - $autoscale->bend);

			$step_red = $dif_red/$autoscale->nodivisions;
			$step_green = $dif_green/$autoscale->nodivisions;
			$step_blue = $dif_blue/$autoscale->nodivisions;

			$bounder_inf = 0;
			$bounder_sup = round(100/$autoscale->nodivisions);

			$scale = new StdClass;
			for ($i=1; $i < $autoscale->nodivisions; $i++) {
				$scale->entry[$i] = new StdClass;
				$scale->entry[$i]->low = $bounder_inf;
				$scale->entry[$i]->high = $bounder_sup;
				$scale->entry[$i]->r = $autoscale->rstart + $i * $step_red;
				$scale->entry[$i]->r = round($scale->entry[$i]->r, 0);
				$scale->entry[$i]->g = $autoscale->gstart + $i * $step_green;
				$scale->entry[$i]->g = round($scale->entry[$i]->g, 0);
				$scale->entry[$i]->b = $autoscale->bstart + $i * $step_blue;
				$scale->entry[$i]->b = round($scale->entry[$i]->b, 0);
				$bounder_inf = $bounder_sup;
				if ($i == ($autoscale->nodivisions-2)) {
					$bounder_sup = 100;
				} else {
		    	$bounder_sup = ($i+2)*round(100/$autoscale->nodivisions);
				}
			}
			return $scale;
		}
	}
}
?>
